#include "command.hpp"
#include <iostream>

icfp::map::map() {
}


icfp::rover::rover()
: waypoint( 0, 0 ) {
}


namespace {
    void accelerate( const icfp::link &link ) {
        link.send( "a;" );
        std::cout << " -- accelerate";
    }
    void brake( const icfp::link &link ) {
        link.send( "b;" );
        std::cout << " -- brake";
    }
    void left( const icfp::link &link ) {
        link.send( "l;" );
        std::cout << " -- left";
    }
    void right( const icfp::link &link ) {
        link.send( "r;" );
        std::cout << " -- right";
    }
}
void icfp::rover::move( const icfp::link &link, icfp::map &map ) {
    double waypoint_bearing = icfp::bearing( position, waypoint );
    double turn_to = icfp::relative_bearing( dir, waypoint_bearing );
    
    std::cout
        << accelerator << steering << " "
        << position.first << ", " << position.second << " "
        << dir << "@" << speed << " "
        << waypoint_bearing << " " << turn_to;
    
    if ( speed < 0.1 )
        accelerate( link );
    else if ( ( turn_to > 60 || turn_to < -60 ) && accelerator != 'b' )
        brake( link );
    else if ( turn_to < 10 && turn_to > -10 && accelerator != 'a' )
        accelerate( link );

    bool evade = false;
    if ( steering == '-' ) {
        for ( std::vector< icfp::obstacle >::const_iterator it( map.obstacles.begin() ); it != map.obstacles.end(); ++it )
            if ( icfp::will_collide( position, dir, it->position, it->radius ) ) {
                left( link );
                evade = true;
            }
    }
    
    if ( evade )
        ; // We're already trying to turn to evade something
    else if ( turn_to < 0 ) {
        if ( turn_to > -3 ) {
            if ( steering == 'l' )
                right( link );
        } else if ( turn_to > -30  ) {
            if ( steering == 'L' )
                right( link );
        } else if ( steering != 'L' )
            left( link );
    } else if ( turn_to > 0 ) {
        if ( turn_to < 3 ) {
            if ( steering == 'r' )
                left( link );
        } else if ( turn_to < 30 ) {
            if ( steering == 'R' )
                left( link );
        } else if ( steering != 'R' )
            right( link );
    }

    std::cout << std::endl;
}


icfp::obstacle::obstacle( std::istream &in ) {
    in >> position.first >> position.second >> radius;
}

